/* wptSignEncDlg.cpp - Sign & encrypt dialog
 *	Copyright (C) 2000-2006 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WinPT; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>
#include <commctrl.h>

#include "resource.h"
#include "wptErrors.h"
#include "wptAgent.h"
#include "wptCrypto.h"
#include "wptGPG.h"
#include "wptCommonCtl.h"
#include "wptKeylist.h"
#include "wptTypes.h"
#include "wptNLS.h"
#include "wptContext.h" /* for passphrase_s */
#include "wptDlgs.h"
#include "wptW32API.h"
#include "wptKeylist.h"
#include "wptVersion.h"
#include "wptGPG.h"
#include "wptRegistry.h"
#include "wptUTF8.h"


/* Encrypt the clipboard data with the recipients from @rset and
   additionally sign the data before @signer as the keyID.
   Return value: 0 on success. */
gpgme_error_t
gpg_clip_sign_encrypt (gpgme_ctx_t ctx, const char *signer, 
		       gpgme_key_t *rset, int opts)
{
    gpgme_error_t err;
    gpgme_data_t plain = NULL;
    gpgme_data_t ciph = NULL;
    gpgme_key_t key = NULL;
 
    if (!signer)
	return gpg_error (GPG_ERR_INV_ARG);
    if (get_pubkey (signer, &key))
	return gpg_error (GPG_ERR_NO_PUBKEY);

    gpgme_set_armor (ctx, 1);
    gpgme_set_textmode (ctx, 1);
    
    err = gpg_data_utf8_new_from_clipboard (&plain, 0, NULL);
    if (err)
	goto leave;   
    err = gpgme_data_new (&ciph);
    if (err)
	goto leave;
    err = gpgme_signers_add (ctx, key);
    if (err)
	goto leave;

    err = gpgme_op_encrypt_sign (ctx, rset, 
				 opts? GPGME_ENCRYPT_ALWAYS_TRUST : (gpgme_encrypt_flags_t)0,
				 plain, ciph);
    if (err)
	goto leave;

    gpg_data_release_to_clipboard (ciph, 1);
    ciph = NULL;

leave:
    if (plain)
	gpgme_data_release (plain);
    if (ciph)
	gpgme_data_release (ciph);
    return err;
}


/* Dialog procedure for clipboard sign + encrypt. */
BOOL CALLBACK 
clip_signenc_dlg_proc (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    static listview_ctrl_t lv = NULL;
    static keylist_t list = NULL;
    gpg_keycache_t kc;
    gpgme_key_t *rset;
    gpgme_error_t err;
    gpgme_ctx_t ctx;
    passphrase_cb_s pwd;
    char *signer = NULL;
    size_t n;
    int force_trust = 0;
    
    switch (msg) {
    case WM_INITDIALOG:
        SetWindowText (dlg, _("Sign & Encrypt"));
	SetDlgItemText (dlg, IDCANCEL, _("&Cancel"));
        kc = keycache_get_ctx (KEYCACHE_PUB);
        lv = keylist_load (GetDlgItem (dlg, IDC_SIGNENC_KEYLIST),
			   kc, NULL, KEYLIST_ENCRYPT_MIN, KEY_SORT_USERID);
	seclist_init (dlg, IDC_SIGNENC_SECLIST, KEYLIST_FLAG_SHORT, &list);
	center_window (dlg, NULL);
	EnableWindow (GetDlgItem (dlg, IDC_SIGNENC_SECLIST), FALSE);
	SetDlgItemText (dlg, IDC_SIGNENC_SELKEY, _("Select key for signing"));
	SetDlgItemText (dlg, IDC_SIGNENC_SECLISTINF, _("Signing key:"));
        SetForegroundWindow (dlg);
        return TRUE;
        
    case WM_DESTROY:
	seclist_destroy (&list);
        if (lv) {
            keylist_delete (lv);
            lv = NULL;
        }
        return FALSE;
        
    case WM_NOTIFY:
        NMHDR *notify;
        notify = (NMHDR *)lparam;
        if (notify && notify->code == NM_DBLCLK
            && notify->idFrom == IDC_SIGNENC_KEYLIST)
            PostMessage (dlg, WM_COMMAND, MAKEWPARAM (IDOK, 0), 0);
        return TRUE;
        
    case WM_COMMAND:
	if (HIWORD (wparam) == BN_CLICKED
	    && LOWORD (wparam) == IDC_SIGNENC_SELKEY) {
	    int enable = IsDlgButtonChecked (dlg, IDC_SIGNENC_SELKEY);
	    EnableWindow (GetDlgItem (dlg, IDC_SIGNENC_SECLIST), enable? TRUE : FALSE);
	}
        switch (LOWORD (wparam)) {
        case IDOK:
            rset = keylist_get_recipients (lv, &force_trust, &n);
	    if (!n) {
                msg_box (dlg, _("You must select at least one key."),
			 _("Sign & Encrypt"), MB_ERR);
                return FALSE;
            }
            if (IsDlgButtonChecked (dlg, IDC_SIGNENC_SELKEY)) {
		gpgme_key_t key;
		const char *s;

		if (seclist_select_key (dlg, IDC_SIGNENC_SECLIST, &key)) {
		    msg_box (dlg, _("No key was selected."), _("Signing"), MB_ERR);
		    safe_free (rset);
                    return FALSE;
		}
		s = key->subkeys->keyid;
		if (s)
		    signer = m_strdup(s+8);
            }
            else {
		signer = get_gnupg_default_key ();
		if (!signer) {
                    msg_box (dlg, _("Could not get default key."), _("Signing"), MB_ERR);
		    safe_free (rset);
                    return FALSE;
                }
            }

	    err = gpgme_new (&ctx);
	    if (err)
		BUG (NULL);
	    set_gpg_passphrase_cb (&pwd, ctx, GPG_CMD_SIGN, dlg, _("Sign & Encrypt"));
            err = gpg_clip_sign_encrypt (ctx, signer, rset, force_trust);
	    release_gpg_passphrase_cb (&pwd);
            safe_free (rset);
	    free_if_alloc (signer);
	    gpgme_release (ctx);
	    if (gpgme_err_code (err) ==  GPG_ERR_BAD_PASSPHRASE)
		agent_del_cache (pwd.keyid);
	    if (err)
		msg_box (dlg, gpgme_strerror (err), _("Sign & Encrypt"), MB_ERR);
	    else {
		show_msg (dlg, 1500, _("GnuPG Status: Finished"));
		EndDialog (dlg, TRUE);
	    }
            return TRUE;
            
        case IDCANCEL:
            EndDialog (dlg, FALSE);
            return FALSE;
        }
        break;
    }
    
    return FALSE;
}
