/* wptSigList.cpp - Listview for key signatures
 *	Copyright (C) 2001-2006 Timo Schulz
 *	Copyright (C) 2005, 2006 g10 Code GmbH
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *  
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>
#include <time.h>

#include "resource.h"
#include "wptGPG.h"
#include "wptCommonCtl.h"
#include "wptKeylist.h"
#include "wptNLS.h"
#include "wptErrors.h"
#include "wptTypes.h"
#include "wptW32API.h"
#include "wptVersion.h"
#include "wptContext.h"


/* Is the given signature an user-id signature? */
#define IS_UID_CERT(_class) ((_class) >= 0x10 && (_class) <= 0x13)


/* Create a signature list control in @lv with the parent window
   given in @ctrl.
   Return value: 0 on success. */
static void
siglist_build (listview_ctrl_t *lv, HWND ctrl)
{
    listview_ctrl_t c;
    struct listview_column_s implist[] = {
	{0, 240, (char *)_("User ID")},
	{1,  50, (char *)_("Valid")},
	{2,  44, (char *)_("Class")},
	{3,  68, (char *)_("Creation")},
	{4,  80, (char *)_("Key ID")},
	{5,  68, (char *)_("Expiration")},
	{6,  56, (char *)_("Algorithm")},
	{0,   0, NULL}
    };

    listview_new (&c, ctrl);
    for  (int j=0; implist[j].fieldname != NULL; j++)
	listview_add_column (c, &implist[j]);
    listview_set_ext_style (c);
    listview_del_all_items (c);
    *lv = c;
}


/* Delete the signature control in @lv. */
void 
siglist_delete (listview_ctrl_t lv)
{
    if (lv) {
	listview_release (lv);
    }
}


/* Indent the string in @old with the level @ilvl.
    Return value: indented string. */
static char*
indent_string (const char *old, int ilvl)
{
    char *p;
    int i;
    
    p = new char[strlen (old) + 1 + ilvl];
    if (!p)
	BUG (NULL);
    for (i = 0; i < ilvl; i++)
	p[i] = ' ';
    strcpy (p+i, old);
    return p;
}


/* Add an item at the given pos @pos with the opaque param @ks. */
int 
add_keysig_item (listview_ctrl_t ctx, int pos, gpgme_key_sig_t ks)
{
    LV_ITEM lvi;

    memset (&lvi, 0, sizeof (lvi));
    lvi.iItem = pos;
    lvi.lParam = (LPARAM)ks;
    lvi.mask = LVIF_PARAM;
    if (ListView_InsertItem (ctx->ctrl, &lvi) != pos)
	return -1;
    return 0;
}


/* Add an userid signature @ks from the userid @uid to the sig list 
   control @lv.
   Return value: 0 on success. */
static int
siglist_add_key (listview_ctrl_t lv, struct native_uid_s *uid, 
		 gpgme_key_sig_t ks, int pos)
{
    struct winpt_key_s key;    
    const char *attr;
    const char *s;
    char t[128];
    int no_uid = 0;
    int rc;

    if (ks && !IS_UID_CERT (ks->sig_class))
	return 0;
    
    rc = add_keysig_item (lv, pos, ks);
    if (rc)
	return rc;

    attr = NULL;
    if (!uid) {
	memset (&key, 0, sizeof (key));
	winpt_get_pubkey (ks->keyid, &key);
	if (key.ext != NULL)
	    attr = key.ext->uids->uid;
    }
    else
	attr = uid->uid;
    if (attr && strlen (attr) > 0) {
	char *p = uid? m_strdup (attr) : indent_string (attr, 2);
	listview_add_sub_item (lv, pos, SL_COL_UID, p);
	free_if_alloc (p);
    }
    else {
	listview_add_sub_item (lv, pos, SL_COL_UID, _("  user ID not found"));
	no_uid = 1;
    }

    /* Return immediately for root items. */
    if (uid)
	return 0;

    /* TRANSLATORS: these strings should be in capital letters. */
    switch (gpgme_err_code (ks->status)) {
    case GPG_ERR_NO_ERROR:     s = _("YES");   break;
    case GPG_ERR_NO_PUBKEY:    s = _("NOKEY"); break;
    case GPG_ERR_BAD_SIGNATURE:s = _("NO");    break;
    default:                   s = _("ERROR"); break;
    }
    if (no_uid)
	s = _("NOKEY");
    listview_add_sub_item (lv, pos, SL_COL_VALID, s);

    switch (ks->sig_class) {
    case 0x10: strcpy (t, " "); break;
    case 0x11: strcpy (t, "1"); break;
    case 0x12: strcpy (t, "2"); break;
    case 0x13: strcpy (t, "3"); break;
    }

    strcat (t, " ");
    if (!ks->exportable)
	strcat (t, "L");
    /* XXX: if gpgme supports non-revocable status, add 'R' */
    listview_add_sub_item (lv, pos, SL_COL_CLASS, t);

    s = get_key_created (ks->timestamp);
    listview_add_sub_item (lv, pos, SL_COL_CREATE, s);

    if (ks->keyid && strlen (ks->keyid) == 16) {
	_snprintf (t, DIM (t) -1, "0x%s", ks->keyid + 8);
	listview_add_sub_item (lv, pos, SL_COL_KEYID, t);
    }

    if (ks->expires > 0) {
	s = get_key_created (ks->expires);
	listview_add_sub_item (lv, pos, SL_COL_EXPIRE, s);
    }

    attr = get_key_pubalgo (ks->pubkey_algo);
    if (attr)
	listview_add_sub_item (lv, pos, 6, (char *)attr);

    return 0;
}


/* Load a signature list with the signatures of the key with
   the keyID @keyid. The parent window is in @ctrl.
   Return value is the handle to the listview control. */
listview_ctrl_t
siglist_load (HWND ctrl, const char *keyid)
{    
    gpgme_key_sig_t s;
    struct native_uid_s *u;
    winpt_key_s key;
    listview_ctrl_t lv;
    int pos = 0;
    int rc = 0;

    siglist_build (&lv, ctrl);
    memset (&key, 0, sizeof (key));
    if (winpt_get_pubkey (keyid, &key))
	BUG (NULL);

    for (u=key.ext->uids; u; u = u->next) {
	siglist_add_key (lv, u, NULL, pos++);
	for (s = u->signatures; s; s = s->next) {
	    if (!IS_UID_CERT (s->sig_class))
		continue;
	    rc = siglist_add_key (lv, NULL, s, pos++);
	    if (rc)
		break;
	}
    }
    if (rc) {
	siglist_delete (lv);
	lv = NULL;
    }
    return lv;
}


/* Integer comparsion of @a and @b. 
   Return values: same as in strcmp. */
static inline int
int_cmp (int a, int b)
{
    if (a == b) return 0;	
    else if (a > b) return 1;
    else return -1;
    return 0;
}


/* Sorting callback for the signature list. */
static int CALLBACK
siglist_cmp_cb (LPARAM first, LPARAM second, LPARAM sortby)
{
    gpgme_key_sig_t a, b;
    int cmpresult=0;

    a = (gpgme_key_sig_t)first;
    b = (gpgme_key_sig_t)second;
    if (!a || !b)
	return -1; /* this indicates that one item is a root item. */

    switch (sortby) {
    case KEY_SORT_KEYID: 
	cmpresult = strcmp (a->keyid, b->keyid);
	break;

    case KEY_SORT_CREATED:
	cmpresult = int_cmp (a->timestamp, b->timestamp);
	break;

    case KEY_SORT_ALGO:
	cmpresult = int_cmp (a->pubkey_algo, b->pubkey_algo);
	break;

    case KEY_SORT_VALIDITY:
	cmpresult = int_cmp (a->status, b->status);
	break;

    case SIG_SORT_EXPIRE:
	cmpresult = int_cmp (a->expires, a->expires);
	break;

    case SIG_SORT_CLASS:
	cmpresult = int_cmp (a->sig_class, b->sig_class);
	break;
    }
    return cmpresult;
}


/* Sort the signature list @sigl by @sortby. */
void
siglist_sort (listview_ctrl_t sigl, int sortby)
{
    listview_sort_items (sigl, sortby, siglist_cmp_cb);
}
